/*
 * PCG Random Number Generation for C++
 *
 * Copyright 2014 Melissa O'Neill <oneill@pcg-random.org>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * For additional information about the PCG random number generation scheme,
 * including its license and other licensing options, visit
 *
 *     http://www.pcg-random.org
 */

/*
 * This program 
 *   - reads in the stored state of a pcg32_k16384 RNG (from standard input)
 *   - advances by an amount specified on the command line
 *   - writes 64 KB of data from the RNG to the file 'data.raw'
 */

#include <cstdio>
#include <cstddef>
#include <cstdint>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <random>

#include "pcg_random.hpp"

constexpr size_t MAX_SIZE = 64 * 1024;

char buffer[MAX_SIZE];

int main(int argc, char** argv)
{
    pcg32_k16384 rng;
    std::cin >> rng;

    size_t magnifier = 1;
    for (size_t i = 1; i < argc; ++i) {
        uint64_t adv = strtoull(argv[i], nullptr, 0);
        std::cout << "Advancing by " << adv << " * " << magnifier << std::endl;
        for (size_t j = 0; j < magnifier; ++j)
            rng.advance(adv);
        magnifier *= 256;
    }

    // This way of converting to characters is a little clumsy, but it
    // is portable and doesn't depend on the endian of the machine.
    std::cout << "Generating " << MAX_SIZE/4 << " 'random' numbers\n";
    uint32_t randval;
    for (int i = 0; i < MAX_SIZE; ++i) {
        if (i % 4 == 0)
            randval = rng();
        else
            randval >>= 8;
        buffer[i] = char(randval);
    }
    
    std::cout << "Writing " << MAX_SIZE << " bytes to data.raw\n";
    std::ofstream partyFile ("data.raw", std::ios::out | std::ios::binary);
    partyFile.write(buffer, MAX_SIZE);

    return 0;
}
